import java.io.*;
import java.util.*;
import javax.microedition.io.*;
import com.nttdocomo.ui.*;

/**
 *	Q[LoXNXB
 */
public class DynamoCanvas extends Canvas {
	private Screen screen = null;
	private Picture picture = null;
	private ShortTimer shortTimer = null;
	private static Random random = new Random();

	/** s[X */
	private int[] posPieces = new int[PIECE_NUM * 2];
	/** Xe[WƂ̃xXg^C(P: msec) */
	private int[] bestTimes = new int[Picture.TYPE_NUM];
	/** oߎ(P: msec) */
	private int progressTime = 0;

	private static final int FRAME_INTERVAL = 100;

	private static final int DIVIDED_NUM = 4;
	private static final int PIECE_NUM = DIVIDED_NUM * DIVIDED_NUM;

	private static final int MAP_WIDTH  = 8;
	private static final int MAP_HEIGHT = 8;

	private static final int[] tablePieceMap = {
		-1, -1, -1, -1, -1, -1, -1, -1,
		-1, -1, 16, 17, 18, 19, -1, -1,
		-1, 24,  0,  1,  2,  3, 28, -1,
		-1, 25,  4,  5,  6,  7, 29, -1,
		-1, 26,  8,  9, 10, 11, 30, -1,
		-1, 27, 12, 13, 14, 15, 31, -1,
		-1, -1, 20, 21, 22, 23, -1, -1,
		-1, -1, -1, -1, -1, -1, -1, -1,
	};

	private static final int[] tableVectors = {
		1,				// E
		-MAP_WIDTH + 1,	// E
		-MAP_WIDTH,		// 
		-MAP_WIDTH - 1,	// 
		-1,				// 
		MAP_WIDTH - 1,	// 
		MAP_WIDTH,		// 
		MAP_WIDTH + 1,	// E
	};

	/**
	 *	Q[LoXIuWFNg𐶐B
	 */
	public DynamoCanvas() {

		screen = new Screen(this);

		// xXg^Cf[^̏s
		loadBestTimes(bestTimes);
		saveBestTimes(bestTimes);

		// s[X̏s
		for (int i = 0; i < posPieces.length; i++)
			posPieces[i] = -1;
	}

	/**
	 *	C[vsB
	 */
	public void run() {

		// V[g^C}[쐬ĊJn
		startShortTimer();

		// ^CgʏԂɂ
		changeState(stateTitle);

		// C[v
		for (;;) {
			final long timeWaitEnd = System.currentTimeMillis() + FRAME_INTERVAL;

			stateCurrent.doTick();

			do {
				try {
					Thread.yield();
				} catch (Exception e) {
					System.out.println(e.toString());
					IApplication.getCurrentApp().terminate();
				}
			} while (System.currentTimeMillis() < timeWaitEnd);
		}
	}

	public void paint(Graphics g) {
	}

	public void processEvent(int type, int param) {
		if (type == Display.TIMER_EXPIRED_EVENT) {
			/*
			 *	L[s[gpV[g^C}[Cxg̏
			 */
			// ݉ĂL[擾
			final int keyBits = getKeypadState();

			// ֘ÃL[Ăꍇ́AL[s[gs
			if ((keyBits & ((1 << Display.KEY_LEFT) | (1 << Display.KEY_UP) | (1 << Display.KEY_RIGHT) | (1 << Display.KEY_DOWN) |
							(1 << Display.KEY_UPPER_LEFT) | (1 << Display.KEY_UPPER_RIGHT) | (1 << Display.KEY_LOWER_RIGHT) | (1 << Display.KEY_LOWER_LEFT) |
							(1 << Display.KEY_2) | (1 << Display.KEY_4) | (1 << Display.KEY_6) | (1 << Display.KEY_8) |
							(1 << Display.KEY_1) | (1 << Display.KEY_3) | (1 << Display.KEY_7) | (1 << Display.KEY_9))) != 0) {
				// L[s[gpJE^𑝉
				counterDirKeyPress++;

				// Ԃ3ȏAꍇ́AL[s[gs
				if (counterDirKeyPress > 3) {
					// ֘ÃL[͂㉺E4̑gݍ킹ɓ
					int keyBitsFilterd = 0;
					// 
					if ((keyBits & ((1 << Display.KEY_LEFT) | (1 << Display.KEY_UPPER_LEFT) | (1 << Display.KEY_LOWER_LEFT) |
									(1 << Display.KEY_1) | (1 << Display.KEY_4) | (1 << Display.KEY_7))) != 0) {
						keyBitsFilterd |= (1 << Display.KEY_LEFT);
					}
					// 
					if ((keyBits & ((1 << Display.KEY_UP) | (1 << Display.KEY_UPPER_LEFT) | (1 << Display.KEY_UPPER_RIGHT) |
									(1 << Display.KEY_1) | (1 << Display.KEY_2) | (1 << Display.KEY_3))) != 0) {
						keyBitsFilterd |= (1 << Display.KEY_UP);
					}
					// E
					if ((keyBits & ((1 << Display.KEY_RIGHT) | (1 << Display.KEY_UPPER_RIGHT) | (1 << Display.KEY_LOWER_RIGHT) |
									(1 << Display.KEY_3) | (1 << Display.KEY_6) | (1 << Display.KEY_9))) != 0) {
						keyBitsFilterd |= (1 << Display.KEY_RIGHT);
					}
					// 
					if ((keyBits & ((1 << Display.KEY_DOWN) | (1 << Display.KEY_LOWER_LEFT) | (1 << Display.KEY_LOWER_RIGHT) |
									(1 << Display.KEY_7) | (1 << Display.KEY_8) | (1 << Display.KEY_9))) != 0) {
						keyBitsFilterd |= (1 << Display.KEY_DOWN);
					}

					// ㉺E4̑gݍ킹8ɕϊ
					final int key;
					switch (keyBitsFilterd) {
					case (1 << Display.KEY_LEFT):	// 
					case (1 << Display.KEY_LEFT) | (1 << Display.KEY_UP) | (1 << Display.KEY_DOWN):		// EE
						key = Display.KEY_LEFT;
						break;
					case (1 << Display.KEY_UP):		// 
					case (1 << Display.KEY_UP) | (1 << Display.KEY_LEFT) | (1 << Display.KEY_RIGHT):	// EEE
						key = Display.KEY_UP;
						break;
					case (1 << Display.KEY_RIGHT):	// E
					case (1 << Display.KEY_RIGHT) | (1 << Display.KEY_UP) | (1 << Display.KEY_DOWN):	// EEE
						key = Display.KEY_RIGHT;
						break;
					case (1 << Display.KEY_DOWN):	// 
					case (1 << Display.KEY_DOWN) | (1 << Display.KEY_LEFT) | (1 << Display.KEY_RIGHT):	// EEE
						key = Display.KEY_DOWN;
						break;
					case (1 << Display.KEY_LEFT) | (1 << Display.KEY_UP):		// 
						key = Display.KEY_UPPER_LEFT;
						break;
					case (1 << Display.KEY_RIGHT) | (1 << Display.KEY_UP):		// E
						key = Display.KEY_UPPER_RIGHT;
						break;
					case (1 << Display.KEY_LEFT) | (1 << Display.KEY_DOWN):		// 
						key = Display.KEY_LOWER_LEFT;
						break;
					case (1 << Display.KEY_RIGHT) | (1 << Display.KEY_DOWN):	// E
						key = Display.KEY_LOWER_RIGHT;
						break;
					default:	// ㉺E
						key = 0;
						break;
					}

					// L[s[gɂ[IȃL[Cxg𑗐M
					if (key != 0) {
						if (stateCurrent != null) {
							stateCurrent.sendEvent(Display.KEY_PRESSED_EVENT, key);
						}
					}
				}

			} else {
				// ֘ÃL[ĂȂꍇ́AL[s[gpJE^
				counterDirKeyPress = 0;
			}

		} else if (type == Display.KEY_PRESSED_EVENT) {
			/*
			 *	L[Cxg̏
			 */
			if (param == Display.KEY_SOFT2) {
				// \tgL[2ꂽꍇ́AiAvI
				IApplication.getCurrentApp().terminate();

			} else {
				// L[Cxg𑗐M(L[1`9́AΉL[ɕϊ)
				final int key = (Display.KEY_1 <= param && param <= Display.KEY_9) ? tableKeyConvert[param - Display.KEY_0] : param;
				if (stateCurrent != null) {
					stateCurrent.sendEvent(type, key);
				}
			}

		} else if (type == Display.RESUME_VM_EVENT) {
			/*
			 *	W[Cxg̏
			 */
			// V[g^C}[쐬ȂĊJn
			startShortTimer();

			// W[Cxgʒm
			if (stateCurrent != null) {
				stateCurrent.sendEvent(type, param);
			}
		}
	}
	/** L[L[ϊe[u */
	private static final int tableKeyConvert[] = {
		Display.KEY_0,				// KEY_0
		Display.KEY_UPPER_LEFT,		// KEY_1
		Display.KEY_UP,				// KEY_2
		Display.KEY_UPPER_RIGHT,	// KEY_3
		Display.KEY_LEFT,			// KEY_4
		Display.KEY_SELECT,			// KEY_5
		Display.KEY_RIGHT,			// KEY_6
		Display.KEY_LOWER_LEFT,		// KEY_7
		Display.KEY_DOWN,			// KEY_8
		Display.KEY_LOWER_RIGHT,	// KEY_9
	};
	/** L[s[gpJE^ */
	private int counterDirKeyPress;

	/**
	 *	V[g^C}[JnB
	 */
	private void startShortTimer() {
		// V[g^C}[쐬
		if (shortTimer != null) {
			shortTimer.dispose();
			shortTimer = null;
		}

		// V[g^C}[Jn
		shortTimer = ShortTimer.getShortTimer(this, 0, FRAME_INTERVAL, true);
		shortTimer.start();
	}

	private static int getRandom(int max) {return (random.nextInt() >>> 1) % max;}

	/**
	 *	sN`ύXB
	 *	@param	type	sN`̎(TYPE_BREAK_OUTȂ)
	 */
	private void changePicture(int type) {
		// sN`̏Is
		if (picture != null) {
			picture.quit();
		}

		// sN`̐Es
		picture = Picture.getInstance(type);
		picture.init();
	}

	/**
	 *	Xe[W\p擾B
	 *	@return	Xe[W\p
	 */
	private String getStageString() {
		StringBuffer sbStage = new StringBuffer();
		sbStage.append("STAGE")
			.append(picture.getType())
			.append(": ")
			.append(picture.getName());

		return sbStage.toString();
	}

	/**
	 *	XNb`pbhփxXg^Cf[^ށB
	 *	@param	times	xXg^Cf[^
	 */
	private void saveBestTimes(int[] times) {
		try {
			DataOutputStream dos = Connector.openDataOutputStream("scratchpad:///0");

			for (int i = 0; i < Picture.TYPE_NUM; i++) {
				dos.writeInt(Picture.getPictureName(i).hashCode());
				dos.writeInt(times[i]);
			}

			dos.close();

		} catch (Exception e) {
			System.out.println(e.toString());
		}
	}

	/**
	 *	XNb`pbhxXg^Cf[^ǂݍށB
	 *	@param	times	xXg^Cf[^
	 */
	private void loadBestTimes(int[] times) {
		try {
			DataInputStream dis = Connector.openDataInputStream("scratchpad:///0");

			for (int i = 0; i < Picture.TYPE_NUM; i++) {
				try {
					final int hashTmp = dis.readInt();
					final int timeTmp = dis.readInt();

					for (int j = 0; j < Picture.TYPE_NUM; j++) {
						if (Picture.getPictureName(j).hashCode() == hashTmp) {
							times[j] = timeTmp;
							break;
						}
					}

				} catch (Exception e) {
					break;
				}
			}

			dis.close();

		} catch (Exception e) {
			System.out.println(e.toString());
		}
	}

	// eʏԃNX̃IuWFNg
	private StateTitle       stateTitle       = new StateTitle();
	private StateMenuPicture stateMenuPicture = new StateMenuPicture();
	private StatePlay        statePlay        = new StatePlay();
	private StatePlay2       statePlay2       = new StatePlay2();
	private StatePause       statePause       = new StatePause();
	private StateGameOver    stateGameOver    = new StateGameOver();
	private StateFadeOut     stateFadeOut     = new StateFadeOut();

	// ԑJڗp̎Q
	private State stateCurrent  = null;
	private State statePrevious = null;

	/**
	 *	ԑJڂsB
	 *	@param	stateNext	ɑJڂԂւ̎Q
	 */
	private void changeState(State stateNext) {

		statePrevious = stateCurrent;
		stateCurrent  = stateNext;

		stateCurrent.init();
	}

	/**
	 *	ʏԃNX
	 *		eʏԃNX̊NXB
	 */
	public abstract class State {

		/**
		 *	ԂB
		 *	ԑJڎɍŏɌĂ΂邱Ƃz肵ĂB
		 */
		public abstract void init();

		/**
		 *	݂̏Ԃɉĕ`揈sB
		 */
		public abstract void redraw();

		/**
		 *	݂̏Ԃɉĕ`揈sB
		 *	{Iɂ́A100msecƂɌĂ΂邱Ƃz肵ĂB
		 */
		public abstract void doTick();

		/**
		 *	CxgʒmB
		 *	@param	type	Cxg̃^Cv
		 *	@param	param	Cxg̃p[^
		 *	@see	com.nttdocomo.ui.Canvas
		 */
		public void sendEvent(int type, int param) {
			if (type == Display.KEY_PRESSED_EVENT) {
				pushKeyEvent(param);
			} else if (type == Display.RESUME_VM_EVENT) {
				redraw();
			}
		}

		protected final void pushKeyEvent(int key)	{keyLastPressed = key;}

		protected final int popKeyEvent() {
			final int keyTmp = keyLastPressed;
			keyLastPressed = 0;
			return keyTmp;
		}

		private int keyLastPressed;
	}

	/**
	 *	ʏԃNX(^Cg)
	 */
	class StateTitle extends State {

		public StateTitle() {}

		public void init() {
			// s[Xzu
			for (int i = 0; i < PIECE_NUM; i++) {
				posPieces[i]             = i;
				posPieces[i + PIECE_NUM] = -1;
			}

			// ^CgʗpsN`ɕύX
			changePicture(Picture.TYPE_TITLE);

			// \tgL[̃xݒ肷
			setSoftLabel(Frame.SOFT_KEY_1, "");
			setSoftLabel(Frame.SOFT_KEY_2, "Exit");

			// `揈s
			redraw();
		}

		public void redraw() {
			// ʑŜĕ`悷
			screen.lockGrp();
			screen.drawBackground();
			screen.drawLabelString("HIT SELECT KEY", Screen.POS_BOTTOM);
			screen.drawPieces(picture.getImage(), posPieces);
			screen.unlockGrp(true);
		}

		public void doTick() {
			// L[͂擾
			final int key = popKeyEvent();

			// sN`XV
			picture.update();

			// ʕ\XV
			screen.lockGrp();
			screen.drawPieces(picture.getImage(), posPieces);
			screen.unlockGrp(true);

			// L[͂ɉďԑJڂ
			if (key == Display.KEY_SOFT1) {
				changeState(statePlay);
			} else if (key == Display.KEY_ASTERISK) {
				changeState(statePlay2);
			} else if (key != 0) {
				changeState(stateFadeOut);
				stateFadeOut.setNextState(stateMenuPicture);
			}
		}
	}

	/**
	 *	ʏԃNX(sN`I)
	 */
	class StateMenuPicture extends State {

		private int selectedType = 1;
		private boolean flagEnableBlind = false;
		private int dirSlidingH = 0;
		private int offsetX = 0;
		private int localCounter = 0;

		public StateMenuPicture() {}

		public void init() {
			// s[Xzu
			for (int i = 0; i < PIECE_NUM; i++) {
				posPieces[i]             = i;
				posPieces[i + PIECE_NUM] = -1;
			}

			//
			flagEnableBlind = (bestTimes[selectedType] == 0);
			dirSlidingH = 0;
			localCounter = 0;

			// ݑIԂ̃sN`ɕύX
			changePicture(selectedType);

			// \tgL[̃xݒ肷
			setSoftLabel(Frame.SOFT_KEY_1, "Title");
			setSoftLabel(Frame.SOFT_KEY_2, "Exit");

			// `揈s
			redraw();
		}

		public void redraw() {
			// ʑŜĕ`悷
			screen.lockGrp();
			screen.drawBackground();
			screen.drawPieces(picture.getImage(), posPieces);
			screen.drawLabelString("STAGE SELECT", Screen.POS_TOP);
			screen.drawLabelString("HIT SELECT KEY", Screen.POS_BOTTOM);
			screen.drawLabelString("", Screen.POS_LEFT);
			screen.drawLabelString("", Screen.POS_RIGHT);
			screen.drawInfoString(getStageString());
			screen.drawTime(bestTimes[selectedType]);
			screen.unlockGrp(true);
		}

		public void doTick() {
			// L[͂擾
			final int key = popKeyEvent();

			// s[Xzu
			for (int i = 0; i < PIECE_NUM; i++) {
				posPieces[i] = i;
			}

			// L[͂ɉăsN`؂ւ
			boolean flagRedraw = false;
			if (dirSlidingH == 0 &&
				(key == Display.KEY_LEFT || key == Display.KEY_RIGHT)) {
				dirSlidingH = (key == Display.KEY_LEFT) ? -1 : 1;
				offsetX = 0;
			}
			if (dirSlidingH != 0) {
					
				offsetX += dirSlidingH;
				if (offsetX * dirSlidingH >= DIVIDED_NUM) {
					offsetX = -(DIVIDED_NUM - 1) * dirSlidingH;

					do {
						selectedType = (Picture.TYPE_NUM + selectedType + dirSlidingH) % Picture.TYPE_NUM;
					} while (selectedType == Picture.TYPE_TITLE);
					changePicture(selectedType);
					flagRedraw = true;
					flagEnableBlind = (bestTimes[selectedType] == 0);
					localCounter = 0;
				} else if (offsetX == 0) {
					dirSlidingH = 0;
				}

				for (int i = 0; i < PIECE_NUM; i++) {
					final int tmpX = (i % DIVIDED_NUM) + offsetX;
					posPieces[i] = (0 <= tmpX && tmpX < DIVIDED_NUM) ? tmpX + (i / DIVIDED_NUM) * DIVIDED_NUM : -1;
				}
			}

			// NÃXe[W̏ꍇA5bォsN`̈ꕔɋ󗓂\
			if (flagEnableBlind && localCounter >= 50) {
				final int index = (localCounter / 10) % posBlind.length;
				for (int i = 0; i < posBlind[index].length; i++) {
					posPieces[posBlind[index][i]] = -1;
				}
			}

			// sN`XV
			picture.update();

			// ʕ\XV
			screen.lockGrp();
			screen.drawMain(picture.getImage(), posPieces);
			if (flagRedraw) {
				screen.drawInfoString(getStageString());
				screen.drawTime(bestTimes[selectedType]);
			}
			screen.unlockGrp(true);

			// L[͂ɉďԑJڂ
			if (key == Display.KEY_SELECT) {
				changeState(statePlay);
			} else if (key == Display.KEY_ASTERISK) {
				changeState(statePlay2);
			} else if (key == Display.KEY_SOFT1) {
				changeState(stateFadeOut);
				stateFadeOut.setNextState(stateTitle);
			}

			//
			localCounter++;
		}
	}
	private static final int[][] posBlind = {
		{1, 3, 4, 6, 9, 11, 12, 14},
		{0, 2, 5, 7, 8, 10, 13, 15},
	};

	/**
	 *	ʏԃNX(vC)
	 */
	class StatePlay extends State {

		private int posCursorCurrent  = -1;
		private int posCursorSelected = -1;

		public StatePlay() {}

		public void init() {
			if (statePrevious != statePause) {	// |[YԂ畜Aۂɂ͏sȂ
				//
				progressTime = 0;

				// s[Xzu
				for (int i = 0; i < PIECE_NUM; i++) {
					posPieces[i]             = -1;
					posPieces[i + PIECE_NUM] = i;
				}

				// s[X_ɓւ
				for (int i = 0; i < 32; i++) {
					final int index1 = PIECE_NUM + getRandom(PIECE_NUM);
					final int index2 = PIECE_NUM + getRandom(PIECE_NUM);

					final int pieceTmp = posPieces[index1];
					posPieces[index1] = posPieces[index2];
					posPieces[index2] = pieceTmp;
				}

				//
				posCursorSelected = -1;
				posCursorCurrent  = -1;
				for (int i = 0; i < tablePieceMap.length; i++) {
					if (tablePieceMap[i] == 0) {
						posCursorCurrent = i;
						break;
					}
				}

				// sN`
				picture.init();
			}

			// \tgL[̃xݒ肷
			setSoftLabel(Frame.SOFT_KEY_1, "Pause");
			setSoftLabel(Frame.SOFT_KEY_2, "Exit");

			// `揈s
			redraw();
		}

		public void redraw() {
			// ʑŜĕ`悷
			screen.lockGrp();
			screen.drawBackground();
			screen.drawPieces(picture.getImage(), posPieces);
			screen.drawBlanks(posPieces);
			screen.drawInfoString(getStageString());
			screen.drawTime(progressTime);
			screen.unlockGrp(true);
		}

		public void doTick() {
			// L[͂擾
			final int key = popKeyEvent();

			// J[\ʒuۑĂ
			final int posCursorCurrentOld  = posCursorCurrent;
			final int posCursorSelectedOld = posCursorSelected;

			// J[\ړ
			final int dir;
			switch (key) {
			case Display.KEY_RIGHT:
				dir = 0;
				break;
			case Display.KEY_UPPER_RIGHT:
				dir = 1;
				break;
			case Display.KEY_UP:
				dir = 2;
				break;
			case Display.KEY_UPPER_LEFT:
				dir = 3;
				break;
			case Display.KEY_LEFT:
				dir = 4;
				break;
			case Display.KEY_LOWER_LEFT:
				dir = 5;
				break;
			case Display.KEY_DOWN:
				dir = 6;
				break;
			case Display.KEY_LOWER_RIGHT:
				dir = 7;
				break;
			default:
				dir = -1;
				break;
			}
			if (dir >= 0) {
				if (tablePieceMap[posCursorCurrent + tableVectors[dir]] >= 0) {
					posCursorCurrent += tableVectors[dir];
				} else if (tablePieceMap[posCursorCurrent + tableVectors[(dir + 1) % 8]] >= 0) {
					posCursorCurrent += tableVectors[(dir + 1) % 8];
				} else if (tablePieceMap[posCursorCurrent + tableVectors[(dir - 1 + 8) % 8]] >= 0) {
					posCursorCurrent += tableVectors[(dir - 1 + 8) % 8];
				}
			}

			// IL[ꂽꍇAs[XI/s[Xs
			if (key == Display.KEY_SELECT) {
				if (posCursorSelected < 0) {
					// ݂̃J[\ʒũs[XIԂɂ
					posCursorSelected = posCursorCurrent;
				} else {
					// ݂̃J[\ʒũs[XƑIԂ̃s[XƂ
					final int pieceTmp = posPieces[tablePieceMap[posCursorCurrent]];
					posPieces[tablePieceMap[posCursorCurrent]] = posPieces[tablePieceMap[posCursorSelected]];
					posPieces[tablePieceMap[posCursorSelected]] = pieceTmp;

					posCursorSelected = -1;

					//
					if (isComplete(posPieces)) {
						//
						final int type = picture.getType();
						if (bestTimes[type] <= 0 || progressTime < bestTimes[type]) {
							bestTimes[type] = progressTime;
							saveBestTimes(bestTimes);
						}
						//
						changeState(stateGameOver);
						return;
					}
				}
			}

			// L[͂ɉďԑJڂ
			if (key == Display.KEY_SOFT1) {
				changeState(statePause);
				return;
			}

			//
			progressTime += FRAME_INTERVAL;

			// sN`XV
			picture.update();

			// ʕ\XV
			screen.lockGrp();
			if (progressTime % 1000 == 0) {
				screen.drawTime(progressTime);
			}
			screen.drawPieces(picture.getImage(), posPieces);
			if (posCursorCurrent >= 0) {
				screen.drawCursor(tablePieceMap[posCursorCurrent], Screen.CURSOR_TYPE_CURRENT);
			}
			if (posCursorSelected >= 0) {
				screen.drawCursor(tablePieceMap[posCursorSelected], Screen.CURSOR_TYPE_SELECTED);
			}
			if (posCursorCurrentOld != posCursorCurrent && posPieces[tablePieceMap[posCursorCurrentOld]] < 0) {
				screen.drawBlank(tablePieceMap[posCursorCurrentOld]);
			}
			if (posCursorSelectedOld != posCursorSelected && posCursorSelectedOld >= 0) {
				if (posPieces[tablePieceMap[posCursorCurrentOld]] < 0) {
					screen.drawBlank(tablePieceMap[posCursorCurrentOld]);
				}
				if (posPieces[tablePieceMap[posCursorSelectedOld]] < 0) {
					screen.drawBlank(tablePieceMap[posCursorSelectedOld]);
				}
			}
			screen.unlockGrp(true);
		}

		public void sendEvent(int type, int param) {
			if (type == Display.KEY_PRESSED_EVENT) {
				pushKeyEvent(param);
			} else if (type == Display.RESUME_VM_EVENT) {
				//
				changeState(statePause);
			}
		}

		/**
		 *	
		 *	@param	pieces	
		 */
		private boolean isComplete(int[] pieces) {
			for (int i = 0; i < PIECE_NUM; i++) {
				if (pieces[i] != i) {
					return false;
				}
			}
			return true;
		}
	}

	/**
	 *	ʏԃNX(vC(XChpY[h))
	 */
	class StatePlay2 extends State {

		/** 󗓈ʒu */
		private int posBlank;

		public StatePlay2() {}

		public void init() {
			if (statePrevious != statePause) {	// |[YԂ畜Aۂɂ͏sȂ
				// oߎԂ
				progressTime = 0;

				// 󗓈ʒu
				posBlank = -1;
				for (int i = 0; i < tablePieceMap.length; i++) {
					if (tablePieceMap[i] == PIECE_NUM - 1) {
						posBlank = i;
						break;
					}
				}

				// s[Xzu
				for (int i = 0; i < PIECE_NUM; i++) {
					posPieces[i]             = i;
					posPieces[i + PIECE_NUM] = -1;
				}
				posPieces[tablePieceMap[posBlank]] = -1;

				// s[X̓ւs
				do {
					for (int i = 0; i < 32; i++) {
						final int[] keyTmp = {Display.KEY_UP, Display.KEY_DOWN, Display.KEY_LEFT, Display.KEY_RIGHT};
						slidePiece(keyTmp[getRandom(keyTmp.length)]);
					}
				} while (isComplete(posPieces));

				// sN`
				picture.init();
			}

			// \tgL[̃xݒ肷
			setSoftLabel(Frame.SOFT_KEY_1, "Pause");
			setSoftLabel(Frame.SOFT_KEY_2, "Exit");

			// `揈s
			redraw();
		}

		public void redraw() {
			// ʑŜĕ`悷
			screen.lockGrp();
			screen.drawBackground();
			screen.drawMain(picture.getImage(), posPieces);
			screen.drawLabelString("SLIDE PUZZLE MODE", Screen.POS_TOP);
			screen.drawInfoString(getStageString());
			screen.unlockGrp(true);
		}

		public void doTick() {
			// L[͂擾
			final int key = popKeyEvent();

			// 󗓈ʒuۑĂ
			final int posBlankOld = posBlank;

			//
			slidePiece(key);
			if (posBlankOld != posBlank) {
				if (isComplete(posPieces)) {
					posPieces[tablePieceMap[posBlank]] = tablePieceMap[posBlank];
					changeState(stateGameOver);
				}
			}

			// L[͂ɉďԑJڂ
			if (key == Display.KEY_SOFT1) {
				changeState(statePause);
				return;
			}

			//
			progressTime += FRAME_INTERVAL;

			// sN`XV
			picture.update();

			// ʕ\XV
			screen.lockGrp();
			screen.drawTime(progressTime);
			screen.drawMain(picture.getImage(), posPieces);
			screen.unlockGrp(true);
		}

		public void sendEvent(int type, int param) {
			if (type == Display.KEY_PRESSED_EVENT) {
				pushKeyEvent(param);
			} else if (type == Display.RESUME_VM_EVENT) {
				changeState(statePause);	// W[Aɂ͎IɃ|[YԂɈڍs
			}
		}

		private boolean isComplete(int[] pieces) {
			for (int i = 0; i < PIECE_NUM - 1; i++) {	// 16ڂ̃s[X݂͑Ȃ߁A15Ă邩`FbN
				if (pieces[i] != i) {
					return false;
				}
			}
			return true;
		}

		private void slidePiece(int key) {
			// s[XXCh
			final int dir;
			switch (key) {
			case Display.KEY_RIGHT:
				dir = 4;
				break;
			case Display.KEY_UP:
				dir = 6;
				break;
			case Display.KEY_LEFT:
				dir = 0;
				break;
			case Display.KEY_DOWN:
				dir = 2;
				break;
			default:
				dir = -1;
				break;
			}
			if (dir >= 0) {
				final int posTmp = tablePieceMap[posBlank + tableVectors[dir]];
				if (0 <= posTmp && posTmp < PIECE_NUM) {
					posPieces[tablePieceMap[posBlank]] = posPieces[posTmp];
					posPieces[posTmp] = -1;
					posBlank += tableVectors[dir];
				}
			}
		}
	}

	/**
	 *	ʏԃNX(|[Y)
	 */
	class StatePause extends State {

		public StatePause() {}

		public void init() {
			// \tgL[̃xݒ肷
			setSoftLabel(Frame.SOFT_KEY_1, "Menu");
			setSoftLabel(Frame.SOFT_KEY_2, "Exit");

			// `揈s
			redraw();
		}

		public void redraw() {
			// ʑŜĕ`悷
			screen.lockGrp();
			screen.drawBackground();
			screen.drawLabelString("Pause", Screen.POS_CENTER);
			screen.drawLabelString("HIT SELECT KEY", Screen.POS_BOTTOM);
			screen.drawInfoString(getStageString());
			screen.drawTime(progressTime);
			screen.unlockGrp(true);
		}

		public void doTick() {
			// L[͂擾
			final int key = popKeyEvent();

			// L[͂ɉďԑJڂ
			if (key == Display.KEY_SELECT) {
				changeState(statePrevious);
			} else if (key == Display.KEY_SOFT1) {
				changeState(stateFadeOut);
				stateFadeOut.setNextState(stateMenuPicture);
			}
		}
	}

	/**
	 *	ʏԃNX(Q[I[o[)
	 */
	class StateGameOver extends State {

		public StateGameOver() {}

		public void init() {
			// \tgL[̃xݒ肷
			setSoftLabel(Frame.SOFT_KEY_1, "");
			setSoftLabel(Frame.SOFT_KEY_2, "Exit");

			// `揈s
			redraw();
		}

		public void redraw() {
			// ʑŜĕ`悷
			screen.lockGrp();
			screen.drawBackground();
			screen.drawPieces(picture.getImage(), posPieces);
			screen.drawBlanks(posPieces);
			screen.drawLabelString("CONGRATULATIONS!", Screen.POS_TOP);
			screen.drawLabelString("HIT SELECT KEY", Screen.POS_BOTTOM);
			screen.drawInfoString(getStageString());
			screen.drawTime(progressTime);
			screen.unlockGrp(true);
		}

		public void doTick() {
			// L[͂擾
			final int key = popKeyEvent();

			// sN`XV
			picture.update();

			// ʕ\XV
			screen.lockGrp();
			screen.drawPieces(picture.getImage(), posPieces);
			screen.unlockGrp(true);

			// L[͂ɉďԑJڂ
			if (key == Display.KEY_SELECT) {
				changeState(stateFadeOut);
				stateFadeOut.setNextState(stateMenuPicture);
			}
		}
	}

	/**
	 *	ʏԃNX(tF[hAEg)
	 */
	class StateFadeOut extends State {

		private State stateNext = null;
		private int localCounter = 0;

		public StateFadeOut() {}

		public void init() {
			stateNext = statePrevious;
			localCounter = 0;
		}

		public void redraw() {
		}

		public void doTick() {
			// L[͂擾
			final int key = popKeyEvent();

			// ʕ\XV
			Graphics g = getGraphics();
			g.lock();
			g.setColor(Graphics.getColorOfRGB(0, 0, 0));
			for (int y = 0; y < getHeight(); y += 10) {
				g.drawLine(0, y + localCounter, getWidth() - 1, y + localCounter);
			}
			g.unlock(true);

			localCounter++;

			// tF[hAEgꍇ́Ȁ(sN`I)Ɉڍs
			if (localCounter > 10 || key != 0) {
				changeState(stateNext);
			}
		}

		public void sendEvent(int type, int param) {
			if (type == Display.KEY_PRESSED_EVENT) {
				pushKeyEvent(param);
			} else if (type == Display.RESUME_VM_EVENT) {
				changeState(stateNext);	// W[Aɂ͑Ɏ̏ԂɈڍs
			}
		}

		public void setNextState(State state) {
			stateNext = state;
		}
	}
}
