import java.util.*;
import com.nttdocomo.ui.*;

public class PictureAnElectronicBoard extends Picture {

	private static final int LED_WIDTH  = 4;
	private static final int LED_HEIGHT = 4;

	private static final int FIELD_WIDTH  = WIDTH  / LED_WIDTH;
	private static final int FIELD_HEIGHT = HEIGHT / LED_HEIGHT;

	private static final Font font = Font.getFont(Font.SIZE_LARGE);
	private static final int baseline = font.getAscent() + (FIELD_WIDTH - font.getHeight()) / 2;

	private Image imageResource = null;
	private Image imageLetter   = null;

	private int[] pixelsLetter = new int[FIELD_WIDTH * FIELD_HEIGHT];

	private String strSentence = "";
	private int indexSentence;
	private int lengthSentence;

	private int indexLetter;
	private int widthLetter;

	private static final int[] pixelsLightOn = {
		0x603010, 0xa06020, 0x904018, 0x401808,
		0xa06020, 0xffc0a0, 0xa06020, 0x603010,
		0x904018, 0xa06020, 0x904018, 0x603010,
		0x401808, 0x603010, 0x603010, 0x301008,
	};

	protected void initImp() {
		imageResource = Image.createImage(LED_WIDTH, LED_HEIGHT * (1 + FIELD_HEIGHT));

		{
			Graphics g = imageResource.getGraphics();

			int[] pixelsLightOff = new int[pixelsLightOn.length];
			for (int i = 0; i < pixelsLightOff.length; i++) {
				pixelsLightOff[i] = (pixelsLightOn[i] & 0xfcfcfc) >> 2;
			}

			g.setRGBPixels(0, 0,          LED_WIDTH, LED_HEIGHT, pixelsLightOn,  0);
			g.setRGBPixels(0, LED_HEIGHT, LED_WIDTH, LED_HEIGHT, pixelsLightOff, 0);

			for (int y = 1; y < FIELD_HEIGHT; y++) {
				g.copyArea(0, LED_HEIGHT, LED_WIDTH, LED_HEIGHT, 0, LED_HEIGHT * y);
			}
		}

		imageLetter = Image.createImage(FIELD_WIDTH, FIELD_HEIGHT);

		for (int x = 0; x < FIELD_WIDTH; x++) {
			graphics.drawImage(imageResource, LED_WIDTH * x, 0, 0, LED_HEIGHT, LED_WIDTH, LED_HEIGHT * FIELD_HEIGHT);
		}

		setSentence(getNewSentence());
		setLetter();
	}

	protected void quitImp() {
		if (imageResource != null) {
			imageResource.dispose();
			imageResource = null;
		}
		if (imageLetter != null) {
			imageLetter.dispose();
			imageLetter = null;
		}
	}

	protected void updateImp() {

		graphics.copyArea(LED_WIDTH, 0, LED_WIDTH * (FIELD_WIDTH - 1), LED_HEIGHT * FIELD_HEIGHT, -LED_WIDTH, 0);
		graphics.drawImage(imageResource, LED_WIDTH * (FIELD_WIDTH - 1), 0, 0, LED_HEIGHT, LED_WIDTH, LED_HEIGHT * FIELD_HEIGHT);

		for (int y = 0; y < FIELD_HEIGHT; y++) {
			if (pixelsLetter[FIELD_HEIGHT * y + indexLetter] != 0x000000) {
				graphics.drawImage(imageResource, LED_WIDTH * (FIELD_WIDTH - 1), LED_HEIGHT * y, 0, 0, LED_WIDTH, LED_HEIGHT);
			}
		}

		indexLetter++;
		if (indexLetter >= widthLetter) {
			indexSentence++;
			if (indexSentence >= lengthSentence) {
				setSentence(getNewSentence());
			}
			setLetter();
		}
	}

	private void setSentence(String str) {
		strSentence = str;
		indexSentence = 0;
		lengthSentence = strSentence.length();
	}

	private void setLetter() {
		final String strLetter = String.valueOf(strSentence.charAt(indexSentence));

		{
			Graphics g = imageLetter.getGraphics();

			g.setColor(Graphics.getColorOfRGB(0, 0, 0));
			g.fillRect(0, 0, FIELD_WIDTH, FIELD_HEIGHT);

			g.setFont(font);
			g.setColor(Graphics.getColorOfRGB(255, 255, 255));
			g.drawString(strLetter, 0, baseline);

			g.getRGBPixels(0, 0, FIELD_WIDTH, FIELD_HEIGHT, pixelsLetter, 0);
		}

		indexLetter = 0;
		widthLetter = font.stringWidth(strLetter);
	}

	private String getNewSentence() {
		StringBuffer sb = new StringBuffer();
		Calendar cal = Calendar.getInstance();

		switch (getRandom(3)) {
		case 0:
			sb.append("̒[").append(System.getProperty("microedition.platform")).append("łB");
			break;
		case 1:
			sb.append("݂̎").append(cal.get(Calendar.HOUR_OF_DAY)).append("").append(cal.get(Calendar.MINUTE)).append("łB");
			break;
		case 2:
			sb.append("").append(cal.get(Calendar.MONTH) + 1).append("").append(cal.get(Calendar.DATE)).append("łB");
			break;
		}

		return sb.toString();
	}

	public static String getPictureName() {
		return "An Electronic Board";
	}
}
