import com.nttdocomo.ui.*;

public class PictureBounceBalls extends Picture {

	private static final int NUM_BALL = 6;

	private static final int rgbBackground = Graphics.getColorOfRGB(0, 64, 128);

	private int[] colorBall = new int[NUM_BALL];
	private int[] pxBallX   = new int[NUM_BALL];
	private int[] pxBallY   = new int[NUM_BALL];
	private int[] pxBallVX  = new int[NUM_BALL];
	private int[] pxBallVY  = new int[NUM_BALL];

	private static final int[][] pxCircleX = {
		{17, 16, 16, 13, 12, 11, 6, 5, 4, 1, 1, 0, 0, 1, 1, 4, 5, 6, 11, 12, 13, 14, 16, 16, 17, 17},
		{16, 15, 15, 12, 11, 10, 6, 5, 4, 1, 1, 0, 0, 1, 1, 4, 5, 6, 10, 11, 12, 15, 15, 16, 16},
		{14, 13, 13, 11, 10, 9, 5, 4, 3, 1, 1, 0, 0, 1, 1, 3, 4, 5, 9, 10, 11, 13, 13, 14, 14},
		{12, 8, 4, 0, 0, 4, 8, 12, 12},
		{11, 10, 10, 9, 8, 7, 4, 3, 2, 1, 1, 0, 0, 1, 1, 2, 3, 4, 7, 8, 9, 10, 10, 11, 11},
		{10, 9, 9, 8, 7, 6, 4, 3, 2, 1, 1, 0, 0, 1, 1, 2, 3, 4, 6, 7, 8, 9, 9, 10, 10},
	};
	private static final int[][] pxCircleY = {
		{6, 5, 4, 1, 1, 0, 0, 1, 1, 4, 5, 6, 11, 12, 13, 16, 16, 17, 17, 16, 16, 15, 13, 12, 11, 6},
		{6, 5, 4, 1, 1, 0, 0, 1, 1, 4, 5, 6, 10, 11, 12, 15, 15, 16, 16, 15, 15, 12, 11, 10, 6},
		{5, 4, 3, 1, 1, 0, 0, 1, 1, 3, 4, 5, 9, 10, 11, 13, 13, 14, 14, 13, 13, 11, 10, 9, 5},
		{4, 0, 0, 4, 8, 12, 12, 8, 4},
		{4, 3, 2, 1, 1, 0, 0, 1, 1, 2, 3, 4, 7, 8, 9, 10, 10, 11, 11, 10, 10, 9, 8, 7, 4},
		{4, 3, 2, 1, 1, 0, 0, 1, 1, 2, 3, 4, 6, 7, 8, 9, 9, 10, 10, 9, 9, 8, 7, 6, 4},
	};

	protected void initImp() {
		final int tmp = getRandom(NUM_BALL);
		for (int i = 0; i < NUM_BALL; i++) {
			colorBall[i] = Graphics.getColorOfRGB(
				((((tmp + i) % 6 + 1) & 0x01) != 0) ? 255 : 128,
				((((tmp + i) % 6 + 1) & 0x02) != 0) ? 255 : 128,
				((((tmp + i) % 6 + 1) & 0x04) != 0) ? 255 : 128);

			pxBallX[i] = getRandom(WIDTH - pxCircleX[i][0]);
			pxBallY[i] = HEIGHT - pxCircleX[i][0];

			pxBallVX[i] = (getRandom(7) + 2) * ((getRandom(2) == 0) ? 1 : -1);
			pxBallVY[i] = -(getRandom(11) + 6);
		}

		graphics.setColor(rgbBackground);
		graphics.fillRect(0, 0, WIDTH, HEIGHT);
	}

	protected void quitImp() {}

	protected void updateImp() {
		//
		for (int i = 0; i < NUM_BALL; i++) {
			pxBallX[i] += pxBallVX[i];
			pxBallY[i] += pxBallVY[i];

			pxBallVY[i]++;

			if (pxBallX[i] < 0) {
				pxBallX[i] = 0;
				pxBallVX[i] = -pxBallVX[i] - 1;
				if (pxBallVX[i] <= 2) {
					pxBallVX[i] = getRandom(5) + 4;
				}
			} else if (pxBallX[i] >= WIDTH - pxCircleX[i][0]) {
				pxBallX[i] = WIDTH - pxCircleX[i][0];
				pxBallVX[i] = -pxBallVX[i] + 1;
				if (pxBallVX[i] >= -2) {
					pxBallVX[i] = -(getRandom(5) + 4);
				}
			}

			if (pxBallY[i] < 0) {
				pxBallY[i] = 0;
				pxBallVY[i] = -pxBallVY[i];
			} else if (pxBallY[i] >= HEIGHT - pxCircleX[i][0]) {
				pxBallY[i] = HEIGHT - pxCircleX[i][0];
				pxBallVY[i] = -(pxBallVY[i] * 2 / 3);
				if (pxBallVY[i] >= -4) {
					pxBallVY[i] = -(getRandom(5) + 12);
				}
			}
		}

		graphics.setColor(rgbBackground);
		graphics.fillRect(0, 0, WIDTH, HEIGHT);

		for (int i = 0; i < NUM_BALL; i++) {
			graphics.setOrigin(pxBallX[i], pxBallY[i]);

			graphics.setColor(colorBall[i]);
			graphics.fillPolygon(pxCircleX[i], pxCircleY[i], pxCircleX[i].length - 1);
			graphics.setColor(Graphics.getColorOfRGB(0, 0, 0));
			graphics.drawPolyline(pxCircleX[i], pxCircleY[i], pxCircleX[i].length);
		}

		graphics.setOrigin(0, 0);
	}

	public static String getPictureName() {
		return "Bounce Balls";
	}
}
