import com.nttdocomo.ui.*;

public class PictureBreakOut extends Picture {

	private static final int BLOCK_WIDTH  = 16;
	private static final int BLOCK_HEIGHT =  8;

	private static final int PADDLE_WIDTH  = 32;
	private static final int PADDLE_HEIGHT =  4;
	private static final int PADDLE_SPEED  =  4;

	private static final int WAIT_FRAME_GAMEOVER = 8;

	private Image imageResource = null;

	private int[][] fields = new int[16][8];

	private int pxBallX;
	private int pxBallY;
	private int pxBallVX;
	private int pxBallVY;

	private int pxPaddleX;
	private final int pxPaddleY = 112;

	private int numBlocks;
	private int counterGameOver;

	private static final int[] bitsBlockPattern = {
		0x007e007e,
		0x7e007e00,
		0x7e7e7e7e,

		0x0042667e,
		0x40607078,
		0x02060e1e,

		0x4242427e,
		0x18181818,
		0x7e3c1800,
	};

	protected void initImp() {
		imageResource = Image.createImage(BLOCK_WIDTH * 8, BLOCK_HEIGHT + PADDLE_HEIGHT);

		{
			Graphics g = imageResource.getGraphics();

			// wi摜쐬
			g.setColor(Graphics.getColorOfRGB(64, 96, 128));
			g.fillRect(0, 0, BLOCK_WIDTH, BLOCK_HEIGHT);
			g.setColor(Graphics.getColorOfRGB(0, 0, 0));
			g.drawLine(0, 0, BLOCK_WIDTH - 1, 0);
			g.drawLine(0, 4, BLOCK_WIDTH - 1, 4);
			g.drawLine(4, 0, 4, 3);
			g.drawLine(12, 0, 12, 3);
			g.drawLine(0, 4, 0, 7);
			g.drawLine(8, 4, 8, 7);

			// ubN摜쐬
			for (int i = 1; i < 8; i++) {
				final int red   = ((i & 0x01) != 0) ? 192 : 96;
				final int green = ((i & 0x02) != 0) ? 192 : 96;
				final int blue  = ((i & 0x04) != 0) ? 192 : 96;

				g.setOrigin(BLOCK_WIDTH * i, 0);
				g.setColor(Graphics.getColorOfRGB(red - 32, green - 32, blue - 32));
				g.drawRect(0, 0, BLOCK_WIDTH - 1, BLOCK_HEIGHT - 1);
				g.setColor(Graphics.getColorOfRGB(red + 32, green + 32, blue + 32));
				g.drawRect(0, 0, BLOCK_WIDTH - 2, BLOCK_HEIGHT - 2);
				g.setColor(Graphics.getColorOfRGB(red, green, blue));
				g.fillRect(1, 1, BLOCK_WIDTH - 2, BLOCK_HEIGHT - 2);
			}

			// wi摜쐬
			g.setOrigin(0, BLOCK_HEIGHT);
			g.setColor(Graphics.getColorOfRGB(192, 192, 192));
			g.drawRect(0, 0, PADDLE_WIDTH - 1, PADDLE_HEIGHT - 1);
			g.setColor(Graphics.getColorOfRGB(255, 255, 255));
			g.drawRect(0, 0, PADDLE_WIDTH - 2, PADDLE_HEIGHT - 2);
			g.setColor(Graphics.getColorOfRGB(224, 224, 224));
			g.fillRect(1, 1, PADDLE_WIDTH - 2, PADDLE_HEIGHT - 2);

			g.setOrigin(0, 0);
		}

		initGame();

		drawField();
		drawPaddle();
	}

	protected void quitImp() {
		if (imageResource != null) {
			imageResource.dispose();
			imageResource = null;
		}
	}

	protected void updateImp() {

		if (counterGameOver > 0) {
			for (int y = WAIT_FRAME_GAMEOVER - counterGameOver; y < HEIGHT; y += WAIT_FRAME_GAMEOVER) {
				graphics.setColor(Graphics.getColorOfRGB(0, 0, 0));
				graphics.drawLine(0, y, WIDTH, y);
			}

			counterGameOver--;

			if (counterGameOver <= 0) {
				initGame();
				drawField();
			}

			return;
		}

		// phړ
		final int pxPaddleOldX = pxPaddleX;
		if (pxBallY < HEIGHT / 3) {
			if (pxBallX < pxPaddleX) {
				pxPaddleX -= PADDLE_SPEED;
			} else if (pxBallX > pxPaddleX + PADDLE_WIDTH) {
				pxPaddleX += PADDLE_SPEED;
			}
		} else {
			if (pxBallX + pxBallVX * 4 < pxPaddleX + PADDLE_WIDTH / 4) {
				pxPaddleX -= PADDLE_SPEED;
			} else if (pxBallX + pxBallVX * 4 > pxPaddleX + PADDLE_WIDTH - PADDLE_WIDTH / 4) {
				pxPaddleX += PADDLE_SPEED;
			}
		}
		if (pxPaddleX < 0) {
			pxPaddleX = 0;
		} else if (pxPaddleX > WIDTH - PADDLE_WIDTH) {
			pxPaddleX = WIDTH - PADDLE_WIDTH;
		}

		// {[ˏ
		int pxBallTmpX = pxBallX + pxBallVX;
		int pxBallTmpY = pxBallY + pxBallVY;
		if (pxBallTmpX < 0) {
			pxBallTmpX = 0;
			pxBallVX = -pxBallVX;
		} else if (WIDTH - 2 <= pxBallTmpX) {
			pxBallTmpX = WIDTH - 2;
			pxBallVX = -pxBallVX;
		}
		if (pxBallTmpY < 0) {
			pxBallTmpY = 2;
			pxBallVY = -pxBallVY;
		} else if (pxPaddleX <= pxBallTmpX && pxBallTmpX <= pxPaddleX + PADDLE_WIDTH &&
					pxPaddleY - 2 <= pxBallTmpY && pxBallTmpY <= pxPaddleY) {
			pxBallTmpY = pxPaddleY - 2;
			pxBallVX = ((getRandom(3) + 1) * 2) * ((pxBallVX > 0) ? 1 : -1);
			pxBallVY = -pxBallVY;
		} else if (HEIGHT - 2 <= pxBallTmpY) {
			pxBallTmpY = HEIGHT - 2;
			gameOver();
		}

		// ubNj󏈗
		{
			final int fieldBallX = pxBallX / BLOCK_WIDTH;
			final int fieldBallY = pxBallY / BLOCK_HEIGHT;
			final int fieldBallTmpX = pxBallTmpX / BLOCK_WIDTH;
			final int fieldBallTmpY = pxBallTmpY / BLOCK_HEIGHT;
			boolean flagBreak = false;

			// ̃ubNj󂷂
			if (fieldBallTmpY != fieldBallY) {
				if (fields[fieldBallTmpY][fieldBallX] != 0) {
					fields[fieldBallTmpY][fieldBallX] = 0;
					drawBlock(fieldBallX, fieldBallTmpY, 0);
					pxBallTmpY = pxBallY;
					pxBallVY = -pxBallVY;
					numBlocks--;
					flagBreak = true;
				}
			}
			// c̃ubNj󂷂
			if (fieldBallTmpX != fieldBallX) {
				if (fields[fieldBallY][fieldBallTmpX] != 0) {
					fields[fieldBallY][fieldBallTmpX] = 0;
					drawBlock(fieldBallTmpX, fieldBallY, 0);
					pxBallTmpX = pxBallX;
					pxBallVX = -pxBallVX;
					numBlocks--;
					flagBreak = true;
				}
			}
			// Ec̕ɂjłubNȂꍇA΂ߕ̃ubNj󂷂
			if (!flagBreak && fieldBallTmpX != fieldBallX && fieldBallTmpY != fieldBallY) {
				if (fields[fieldBallTmpY][fieldBallTmpX] != 0) {
					fields[fieldBallTmpY][fieldBallTmpX] = 0;
					drawBlock(fieldBallTmpX, fieldBallTmpY, 0);
					pxBallTmpX = pxBallX;
					pxBallTmpY = pxBallY;
					pxBallVX = -pxBallVX;
					pxBallVY = -pxBallVY;
					numBlocks--;
				}
			}
		}

		// phʒũtB[h̍ĕ`
		if (pxPaddleX != pxPaddleOldX) {
			for (int i = 0; i < WIDTH / BLOCK_WIDTH; i++) {
				drawBlock(i, pxPaddleY / BLOCK_HEIGHT, fields[pxPaddleY / BLOCK_HEIGHT][i]);
			}
		}

		// {[ʒũtB[h̍ĕ`
		drawBlock(pxBallX    / BLOCK_WIDTH, pxBallY    / BLOCK_HEIGHT, fields[pxBallY    / BLOCK_HEIGHT][pxBallX    / BLOCK_WIDTH]);
		drawBlock(pxBallX    / BLOCK_WIDTH, pxBallTmpY / BLOCK_HEIGHT, fields[pxBallTmpY / BLOCK_HEIGHT][pxBallX    / BLOCK_WIDTH]);
		drawBlock(pxBallTmpX / BLOCK_WIDTH, pxBallY    / BLOCK_HEIGHT, fields[pxBallY    / BLOCK_HEIGHT][pxBallTmpX / BLOCK_WIDTH]);
		drawBlock(pxBallTmpX / BLOCK_WIDTH, pxBallTmpY / BLOCK_HEIGHT, fields[pxBallTmpY / BLOCK_HEIGHT][pxBallTmpX / BLOCK_WIDTH]);

		// {[ʒu̍XV
		pxBallX = pxBallTmpX;
		pxBallY = pxBallTmpY;

		// {[Eph̕`
		drawBall();
		drawPaddle();

		if (numBlocks <= 0) {
			gameOver();
		}
	}

	private void initGame() {

		clearFields();
		for (int i = 0; i < 3; i++) {
			setBlocks(bitsBlockPattern[getRandom(bitsBlockPattern.length)], getRandom(4) + 1);
		}

		numBlocks = 0;
		for (int y = 0; y < fields.length; y++) {
			for (int x = 0; x < fields[y].length; x++) {
				if (fields[y][x] != 0) {
					numBlocks++;
				}
			}
		}

		pxBallX = 64;
		pxBallY = pxPaddleY - 2;
		pxBallVX = ((getRandom(3) + 1) * 2) * ((getRandom(2) == 0) ? 1 : -1);
		pxBallVY = -4;
		pxPaddleX = (WIDTH - PADDLE_WIDTH) / 2;

		counterGameOver = 0;
	}

	private void gameOver() {
		counterGameOver = WAIT_FRAME_GAMEOVER;
	}

	private void clearFields() {
		for (int y = 0; y < fields.length; y++) {
			for (int x = 0; x < fields[y].length; x++) {
				fields[y][x] = 0;
			}
		}
	}

	private void setBlocks(int bitsPattern, int y) {
		for (int i = 0; i < 4; i++) {
			final int bitsTmp = (bitsPattern >> (i * 8)) & 0xff;
			for (int x = 0; x < 8; x++) {
				if ((bitsTmp & (1 << x)) != 0) {
					fields[y + i][x] = y + i;
				}
			}
		}
	}

	private void drawBlock(int x, int y, int type) {
		graphics.drawImage(imageResource, BLOCK_WIDTH * x, BLOCK_HEIGHT * y, BLOCK_WIDTH * type, 0, BLOCK_WIDTH, BLOCK_HEIGHT);
	}

	private void drawBall() {
		graphics.setColor(Graphics.getColorOfRGB(255, 255, 255));
		graphics.fillRect(pxBallX - 0, pxBallY - 0, 2, 2);
	}

	private void drawField() {
		for (int y = 0; y < fields.length; y++) {
			for (int x = 0; x < fields[y].length; x++) {
				drawBlock(x, y, fields[y][x]);
			}
		}
	}

	private void drawPaddle() {
		graphics.drawImage(imageResource, pxPaddleX, pxPaddleY, 0, BLOCK_HEIGHT, PADDLE_WIDTH, PADDLE_HEIGHT);
	}

	public static String getPictureName() {
		return "Break Out!";
	}
}
