import com.nttdocomo.ui.*;

public class PictureRipples extends Picture {

	private static final int NUM_RIPPLE = 4;
	private static final int NUM_SHADOW = 4;

	private static final int rgbBackground = Graphics.getColorOfRGB(0, 80, 240);
	private static int[] rgbRipples;

	private int[] pxRippleX  = new int[NUM_RIPPLE];
	private int[] pxRippleY  = new int[NUM_RIPPLE];
	private int[] pxRippleVY = new int[NUM_RIPPLE];
	private int[] counterRipple = new int[NUM_RIPPLE];

	{
		rgbRipples = new int[16];
		for (int i = 0; i < rgbRipples.length; i++) {
			rgbRipples[i] = Graphics.getColorOfRGB(
				128 - (128 -   0) * i / 16,
				192 - (192 -  80) * i / 16,
				255 - (255 - 240) * i / 16);
		}
	}

	protected void initImp() {
		for (int i = 0; i < NUM_RIPPLE; i++) {
			counterRipple[i] = -1;
		}

		graphics.setColor(rgbBackground);
		graphics.fillRect(0, 0, WIDTH, HEIGHT);
	}

	protected void quitImp() {}

	protected void updateImp() {
		for (int i = 0; i < NUM_RIPPLE; i++) {
			if (pxRippleVY[i] > 0) {
				pxRippleY[i] += pxRippleVY[i];
				pxRippleVY[i]++;
				if (0 <= pxRippleY[i] &&
					(pxRippleY[i] >= HEIGHT - pxRippleVY[i] || pxRippleVY[i] >= 12 + getRandom(7))) {
					pxRippleVY[i] = -pxRippleVY[i] / 3;
					counterRipple[i] = 0;
				}
			} else if (pxRippleVY[i] < 0) {
				counterRipple[i]++;
				if (counterRipple[i] > 14) {
					pxRippleVY[i] = 0;
					counterRipple[i] = 0;
				}
			} else {
				if (getRandom(8) == 0) {
					pxRippleX[i] = getRandom(WIDTH);
					pxRippleY[i] = -getRandom(HEIGHT) / 2;
					pxRippleVY[i] = 1 + getRandom(3);
				}
			}
		}

		graphics.setColor(rgbBackground);
		graphics.fillRect(0, 0, WIDTH, HEIGHT);

		for (int i = 0; i < NUM_RIPPLE; i++) {
			if (pxRippleVY[i] > 0) {
				graphics.setColor(Graphics.getColorOfRGB(192, 192, 255));
				graphics.drawArc(pxRippleX[i] - 1, pxRippleY[i] - 1, 3, 3, 0, 360);
			} else if (pxRippleVY[i] < 0) {
				for (int j = 0; j < NUM_SHADOW; j++) {
					final int tmpCounter = counterRipple[i] - j;
					if (tmpCounter <= 0 || 16 < tmpCounter) {
						continue;
					}

					final int tmpHeight = -pxRippleVY[i] * tmpCounter * 2;
					final int tmpWidth  = tmpHeight * 3 / 2;
					graphics.setColor(rgbRipples[tmpCounter]);
					graphics.drawArc(pxRippleX[i] - tmpWidth / 2, pxRippleY[i] - tmpHeight / 2, tmpWidth, tmpHeight, 0, 360);
					if (tmpCounter == 1) {
						graphics.drawArc(pxRippleX[i] - 1, pxRippleY[i] - 1 - j * j, 3, 3, 0, 360);
					}
				}
			}
		}
	}

	public static String getPictureName() {
		return "Ripples";
	}
}
