import com.nttdocomo.ui.*;

public class PictureTheFlyingBoards extends Picture {

	private static final int NUM_BOARD = 8;

	private static final int BOARD_WIDTH  = 32;
	private static final int BOARD_HEIGHT = 16;
	private static final int BOARD_SPEED  =  2;

	private static final int rgbBackground = Graphics.getColorOfRGB(176, 144,  64);
	private static final int rgbGeoglyph   = Graphics.getColorOfRGB(144,  96,  32);

	private int[] pxBoardX = new int[NUM_BOARD];
	private int[] pxBoardY = new int[NUM_BOARD];
	private int[] counterBoard = new int[NUM_BOARD];

	private static final int[][] pxGeoglyphPos = {
		{
			  5,  25,  30,  32,  32,  36,  68, 104, 101,  68,  67, 100,  97,  65,  63,  96,
			 93,  61,  60,  92,  89,  58,  57,  88,  85,  54,  53,  84,  81,  51,  50,  80,
			 77,  48,  54,  72,  80,  84,  84,  80,  77,  84,  84,  77,  80,  84,  84,  80,
			 74,  74,  72,  72,  56,  60,  96,  96,  69,  96,  96,  67,  96,  96,  66,  96,
			 96,  65,  96,  96,  65,  96,  96,
		},
		{
			  4,  24,  24,  26,  30,  32,   0,   0,   4,   4,   6,   6,  10,  10,  12,  12,
			 16,  16,  18,  18,  22,  22,  24,  24,  28,  28,  30,  30,  34,  34,  36,  36,
			 40,  40,  46,  46,  42,  42,  44,  44,  46,  46,  48,  48,  50,  50,  52,  52,
			 48,  54,  54,  48,  48,  52,  64,  68,  57,  70,  74,  59,  76,  80,  61,  82,
			 86,  62,  88,  92,  64,  94,  96,
		},
	};
	private static final int[][][][] pxBoardsPos = {
		// pattern 0
		{
			// lower
			{
				{ 0, 31, 31,  0},	// X
				{ 6,  6,  9,  9},	// Y
			},
		},
		// pattern 1
		{
			// upper
			{
				{ 9, 22, 30,  1},	// X
				{ 4,  4,  8,  8},	// Y
			},
			// lower
			{
				{ 1, 30, 30,  1},	// X
				{ 9,  9, 11, 11},	// Y
			},
		},
		// pattern 2
		{
			// upper
			{
				{ 7, 24, 29,  2},	// X
				{ 2,  2, 11, 11},	// Y
			},
			// lower
			{
				{ 2, 29, 28,  3},	// X
				{12, 12, 13, 13},	// Y
			},
		},
		// pattern 3
		{
			// upper
			{
				{ 6, 25, 28,  3},	// X
				{ 1,  1, 13, 13},	// Y
			},
			// lower
			{
				{ 3, 28, 28,  3},	// X
				{14, 14, 14, 14},	// Y
			},
		},
		// pattern 4
		{
			// upper
			{
				{ 4, 27, 27,  4},	// X
				{ 0,  0, 15, 15},	// Y
			},
		},
		// pattern 5
		{
			// upper
			{
				{ 3, 28, 28,  3},	// X
				{ 1,  1,  1,  1},	// Y
			},
			// lower
			{
				{ 3, 28, 25,  6},	// X
				{ 2,  2, 14, 14},	// Y
			},
		},
		// pattern 6
		{
			// upper
			{
				{ 3, 28, 29,  2},	// X
				{ 2,  2,  3,  3},	// Y
			},
			// lower
			{
				{ 2, 29, 24,  7},	// X
				{ 4,  4, 13, 13},	// Y
			},
		},
		// pattern 7
		{
			// upper
			{
				{ 1, 30, 30,  1},	// X
				{ 4,  4,  6,  6},	// Y
			},
			// lower
			{
				{ 1, 30, 22,  9},	// X
				{ 7,  7, 11, 11},	// Y
			},
		},
	};

	private static final int[][] rgbBoards = {
		// pattern 0 & 4
		{
			Graphics.getColorOfRGB( 96,  96,  96),
		},
		// pattern 1 & 5
		{
			Graphics.getColorOfRGB(128, 128, 128),	// upper
			Graphics.getColorOfRGB( 64,  64,  64),	// lower
		},
		// pattern 2 & 6
		{
			Graphics.getColorOfRGB(160, 160, 160),	// upper
			Graphics.getColorOfRGB( 32,  32,  32),	// lower
		},
		// pattern 3 & 7
		{
			Graphics.getColorOfRGB(128, 128, 128),	// upper
			Graphics.getColorOfRGB(  0,   0,   0),	// lower
		},
	};

	protected void initImp() {
		for (int i = 0; i < NUM_BOARD; i++) {
			int x = 0;
			int y = 0;
			do {
				x = getRandom(WIDTH  - BOARD_WIDTH);
				y = getRandom(HEIGHT - BOARD_HEIGHT);
			} while (!isCollisionless(x, y));

			pxBoardX[i] = x;
			pxBoardY[i] = y;
			counterBoard[i] = getRandom(8);
		}

		graphics.setColor(rgbBackground);
		graphics.fillRect(0, 0, WIDTH, HEIGHT);
	}

	protected void quitImp() {}

	protected void updateImp() {
		for (int i = 0; i < NUM_BOARD; i++) {
			if (counterBoard[i] >= 0) {
				pxBoardY[i] += BOARD_SPEED;
				counterBoard[i]++;
				if (pxBoardY[i] > HEIGHT) {
					counterBoard[i] = -1;
				}
			} else {
				if (getRandom(10) == 0) {
					final int x = getRandom(WIDTH - BOARD_WIDTH);
					final int y = -BOARD_HEIGHT;
					if (isCollisionless(x, y)) {
						pxBoardX[i] = x;
						pxBoardY[i] = y;
						counterBoard[i] = getRandom(8);
					}
				}
			}
		}

		graphics.setColor(rgbBackground);
		graphics.fillRect(0, 0, WIDTH, HEIGHT);

		graphics.setOrigin(12, counter % (HEIGHT * 2) - HEIGHT);
		graphics.setColor(rgbGeoglyph);
		graphics.drawPolyline(pxGeoglyphPos[0], pxGeoglyphPos[1], 0, pxGeoglyphPos[0].length);
		graphics.drawPolyline(pxGeoglyphPos[1], pxGeoglyphPos[0], 0, pxGeoglyphPos[0].length);
//		graphics.setOrigin(0, 0);

		for (int i = 0; i < NUM_BOARD; i++) {
			if (counterBoard[i] < 0) {
				continue;
			}
			graphics.setOrigin(pxBoardX[i], pxBoardY[i]);
			final int pattern = counterBoard[i] % 8;
//			final int pattern = (counter + i) % 8;
			for (int j = 0; j < pxBoardsPos[pattern].length; j++) {
				graphics.setColor(rgbBoards[pattern % rgbBoards.length][j]);
				graphics.fillPolygon(pxBoardsPos[pattern][j][0], pxBoardsPos[pattern][j][1], 0, pxBoardsPos[pattern][j][0].length);
			}
		}
		graphics.setOrigin(0, 0);
	}

	private boolean isCollisionless(int x, int y) {
		for (int i = 0; i < NUM_BOARD; i++) {
			if (Math.abs(x - pxBoardX[i]) < BOARD_WIDTH && Math.abs(y - pxBoardY[i]) < BOARD_HEIGHT) {
				return false;
			}
		}

		return true;
	}

	public static String getPictureName() {
		return "The Flying Boards";
	}
}
