import com.nttdocomo.ui.*;

public class PictureTheGameOfLife extends Picture {

	private static final int CELL_WIDTH  = 8;
	private static final int CELL_HEIGHT = 8;

	private static final int FIELD_WIDTH  = WIDTH  / CELL_WIDTH;
	private static final int FIELD_HEIGHT = HEIGHT / CELL_HEIGHT;

	private Image imageResource = null;

	private int[][] fields = new int[2][(1 + FIELD_WIDTH + 1) * (1 + FIELD_HEIGHT + 1)];
	private int fieldIndex;

	private int turnProgress;
	private int turnChangeless;
	private int numAliveCell;

	protected void initImp() {
		imageResource = Image.createImage(CELL_WIDTH * 4, CELL_HEIGHT);

		{
			Graphics g = imageResource.getGraphics();

			// wi摜쐬
			g.setColor(Graphics.getColorOfRGB(160, 160, 160));
			g.fillRect(0, 0, CELL_WIDTH, CELL_HEIGHT);
			g.setColor(Graphics.getColorOfRGB(96, 96, 96));
			g.drawRect(0, 0, CELL_WIDTH - 1, CELL_HEIGHT - 1);

			for (int i = 1; i < 4; i++) {
				final int lumi = 192 + i * 16;
				g.setColor(Graphics.getColorOfRGB(lumi, lumi, lumi));
				g.fillRect(CELL_WIDTH * i, 0, CELL_WIDTH, CELL_HEIGHT);
				g.setColor(Graphics.getColorOfRGB(lumi - 32, lumi - 32, lumi - 32));
				g.drawRect(CELL_WIDTH * i, 0, CELL_WIDTH - 1, CELL_HEIGHT - 1);
			}
		}

		graphics.setColor(Graphics.getColorOfRGB(255, 255, 255));
		graphics.fillRect(0, 0, WIDTH, HEIGHT);

		//
		initField();
	}

	protected void quitImp() {
		if (imageResource != null) {
			imageResource.dispose();
			imageResource = null;
		}
	}

	protected void updateImp() {
		progressField();
		if (turnProgress >= 150 || turnChangeless >= 10) {
			initField();
		}

		drawAllCells();
	}

	private void initField() {
		turnProgress   = 0;
		turnChangeless = 0;
		numAliveCell = 0;

		fieldIndex = 0;
		final int[] fieldsCurrent = fields[fieldIndex];

		//
		for (int i = 0; i < fields.length; i++) {
			for (int j = 0; j < fields[i].length; j++) {
				fields[i][j] = 4;
			}
		}

		//
		for (int y = 0; y < FIELD_HEIGHT; y++) {
			for (int x = 0; x < FIELD_WIDTH; x++) {
				final int indexBase = (x + 1) + (y + 1) * (1 + FIELD_WIDTH + 1);
				if (getRandom(4) == 0) {
					fieldsCurrent[indexBase] = 0;
					numAliveCell++;
				} else {
					fieldsCurrent[indexBase] = 4;
				}
			}
		}
	}

	private void progressField() {

		final int[] fieldsCurrent = fields[fieldIndex];
		final int[] fieldsNext    = fields[1 - fieldIndex];

		final int numAliveCellOld = numAliveCell;
		numAliveCell = 0;

		//
		for (int i = 0; i < fieldsNext.length; i++) {
			fieldsNext[i] = 4;
		}

		//
		for (int y = 0; y < FIELD_HEIGHT; y++) {
			for (int x = 0; x < FIELD_WIDTH; x++) {
				final int indexBase = (x + 1) + (y + 1) * (1 + FIELD_WIDTH + 1);
				int numAliveAround = 0;
				for (int i = 0; i < indexOffsets.length; i++) {
					if (fieldsCurrent[indexBase + indexOffsets[i]] == 0) {
						numAliveAround++;
					}
				}

				if (fieldsCurrent[indexBase] == 0) {
					if (numAliveAround == 2 || numAliveAround == 3) {
						fieldsNext[indexBase] = 0;
						numAliveCell++;
					} else {
						fieldsNext[indexBase] = 1;
					}
				} else {
					if (numAliveAround == 3) {
						fieldsNext[indexBase] = 0;
						numAliveCell++;
					} else {
						fieldsNext[indexBase] = fieldsCurrent[indexBase] + 1;
						if (fieldsNext[indexBase] > 4) {
							fieldsNext[indexBase] = 4;
						}
					}
				}
			}
		}

		fieldIndex = 1 - fieldIndex;

		turnProgress++;
		if (numAliveCell == numAliveCellOld) {
			turnChangeless++;
		} else {
			turnChangeless = 0;
		}
	}
	private static final int[] indexOffsets = {
		-(1 + FIELD_WIDTH + 1) - 1,	-(1 + FIELD_WIDTH + 1),	-(1 + FIELD_WIDTH + 1) + 1,
		-1,							/* 0, */				1,
		(1 + FIELD_WIDTH + 1) - 1,	(1 + FIELD_WIDTH + 1),	(1 + FIELD_WIDTH + 1) + 1,
	};

	private void drawAllCells() {
		//
		graphics.setColor(Graphics.getColorOfRGB(255, 255, 255));
		graphics.fillRect(0, 0, WIDTH, HEIGHT);

		//
		final int[] fieldsCurrent = fields[fieldIndex];
		for (int y = 0; y < FIELD_HEIGHT; y++) {
			for (int x = 0; x < FIELD_WIDTH; x++) {
				final int indexBase = (x + 1) + (y + 1) * (1 + FIELD_WIDTH + 1);
				if (fieldsCurrent[indexBase] < 4) {
					drawCell(x, y, fieldsCurrent[indexBase]);
				}
			}
		}
	}

	private void drawCell(int x, int y, int type) {
		graphics.drawImage(imageResource, CELL_WIDTH * x, CELL_HEIGHT * y, CELL_WIDTH * type, 0, CELL_WIDTH, CELL_HEIGHT);
	}

	public static String getPictureName() {
		return "The Game of Life";
	}
}
