import com.nttdocomo.ui.*;

public class PictureTitle extends Picture {

	private static final int STATE_MOVE = 1;
	private static final int STATE_STAY = 2;

	private static final int rgbBackground = Graphics.getColorOfRGB(0, 0, 0);
	private static final int rgbLogo = Graphics.getColorOfRGB(0, 64, 255);

	private int state;
	private int type;
	private int localCounter;

	private int[][] rgbStars = new int[2][16];
	private int[][] pxStarsX = new int[2][16];
	private int[][] pxStarsY = new int[2][16];

	private int[] pxLetterX = new int[pxLetters.length];
	private int[] pxLetterY = new int[pxLetters.length];

	private static final int[][][][] pxLetters = {
		// "D"
		{
			// outer
			{
				{ 0, 15, 20, 15,  0,  0},	// X
				{ 0,  0, 10, 20, 20,  0},	// Y
			},
			// inner
			{
				{ 5, 11, 15, 11,  5,  5},	// X
				{ 2,  2, 10, 18, 18,  2},	// Y
			},
		},
		// "y"
		{
			{
				{ 0,  5,  7, 10, 15,  5,  0,  4,  0},	// X
				{ 5,  5, 12,  5,  5, 25, 25, 17,  5},	// Y
			},
		},
		// "n"
		{
			{
				{ 0, 10, 15, 15, 10, 10,  5,  5,  0,  0},	// X
				{ 5,  5,  7, 20, 20,  7,  7, 20, 20,  5},	// Y
			},
		},
		// "a"
		{
			// outer
			{
				{ 0, 10, 15, 15,  5,  0,  0,  5, 10, 10,  0,  0},	// X
				{ 5,  5,  7, 20, 20, 18, 12, 10, 10,  7,  7,  5},	// Y
			},
			// inner
			{
				{ 5, 10, 10,  5,  5},	// X
				{12, 12, 18, 18, 12},	// Y
			},
		},
		// "-"
		{
			{
				{ 0, 10, 10,  0,  0},	// X
				{ 8,  8, 10, 10,  8},	// Y
			},
		},
		// "M"
		{
			{
				{ 0, 15, 20, 20, 15, 15, 12, 12,  8,  8,  5,  5,  0,  0},	// X
				{ 0,  0,  2, 20, 20,  2,  2, 15, 15,  2,  2, 20, 20,  0},	// Y
			},
		},
		// "o"
		{
			// outer
			{
				{ 0,  5, 10, 15, 15, 10,  5,  0,  0},	// X
				{ 7,  5,  5,  7, 18, 20, 20, 18,  7},	// Y
			},
			// inner
			{
				{ 5, 10, 10,  5,  5},	// X
				{ 7,  7, 18, 18,  7},	// Y
			},
		},
	};

	private static final int[] pxLetterPosX = {2, 22, 39, 56, 74, 87, 109};

	protected void initImp() {
		//
		state = STATE_MOVE;
		type = getRandom(2);
		localCounter = 0;

		//
		for (int i = 0; i < pxStarsX.length; i++) {
			for (int j = 0; j < pxStarsX[i].length; j++) {
				rgbStars[i][j] = Graphics.getColorOfRGB(
					127 + (i * 64) + (((j & 0x01) != 0) ? 64 : 0),
					127 + (i * 64) + (((j & 0x02) != 0) ? 64 : 0),
					127 + (i * 64) + (((j & 0x04) != 0) ? 64 : 0));

				pxStarsX[i][j] = getRandom(WIDTH);
				pxStarsY[i][j] = j * HEIGHT / pxStarsX[i].length;
			}
		}

		//
		graphics.setColor(rgbBackground);
		graphics.fillRect(0, 0, WIDTH, HEIGHT);
	}

	protected void quitImp() {}

	protected void updateImp() {

		//
		for (int i = 0; i < pxStarsX.length; i++) {
			for (int j = 0; j < pxStarsX[i].length; j++) {
				if (pxStarsY[i][j] < HEIGHT) {
					pxStarsY[i][j] += (i == 0) ? 1 : 2;
				} else {
					pxStarsX[i][j] = getRandom(WIDTH);
					pxStarsY[i][j] = 0;
				}
			}
		}

		//
		final int lumi;
		if (state == STATE_STAY) {
			final int tmp = 240 - localCounter * 16;
			lumi = (tmp > 0) ? tmp : 0;
		} else {
			lumi = 0;
		}

		//
		graphics.setColor(Graphics.getColorOfRGB(lumi, lumi, lumi));
		graphics.fillRect(0, 0, WIDTH, HEIGHT);

		//
		for (int i = 0; i < pxStarsX.length; i++) {
			for (int j = 0; j < pxStarsX[i].length; j++) {
				graphics.setColor(rgbStars[i][j]);
				graphics.setPixel(pxStarsX[i][j], pxStarsY[i][j]);
			}
		}

		//
		final boolean flagChangeState;
		switch (state) {
		case STATE_MOVE:
		default:
			switch (type) {
			case 0:
			default:
				flagChangeState = drawMovingLogo1(localCounter);
				break;
			case 1:
				flagChangeState = drawMovingLogo2(localCounter);
				break;
			}
			break;

		case STATE_STAY:
			switch (type) {
//			case 0:
			default:
				flagChangeState = drawStayingLogo1(localCounter);
				break;
//			case 1:
//				flagChangeState = drawStayingLogo2(localCounter);
//				break;
			}
			break;
		}

		localCounter++;

		//
		if (flagChangeState) {
			state = (state == STATE_MOVE) ? STATE_STAY : STATE_MOVE;
			type = getRandom(2);
			localCounter = 0;
		}
	}

	private boolean drawMovingLogo1(int frame) {
		for (int i = 0; i < pxLetters.length; i++) {
			pxLetterX[i] = frame * pxLetterPosX[i] / 20;
			pxLetterY[i] = 48;
		}

		graphics.setColor(rgbLogo);
		for (int i = 0; i < pxLetters.length; i++) {
			graphics.setOrigin(pxLetterX[i], pxLetterY[i]);
			for (int j = 0; j < pxLetters[i].length; j++) {
				graphics.drawPolyline(pxLetters[i][j][0], pxLetters[i][j][1], 0, pxLetters[i][j][0].length);
			}
		}
		graphics.setOrigin(0, 0);

		return (frame >= 20);
	}

	private boolean drawMovingLogo2(int frame) {
		for (int i = 0; i < pxLetters.length; i++) {
			final int tmp = frame - i * 4;
			if (0 <= tmp && tmp < 7) {
				graphics.setColor(Graphics.getColorOfRGB(240 - tmp * 32, 240 - tmp * 24, 240 - tmp * 16));
				graphics.setOrigin(pxLetterPosX[i], 48);
				for (int j = 0; j < pxLetters[i].length; j++) {
					graphics.drawPolyline(pxLetters[i][j][0], pxLetters[i][j][1], 0, pxLetters[i][j][0].length);
				}
			}
		}
		graphics.setOrigin(0, 0);

		return (frame >= 32);
	}

	private boolean drawStayingLogo1(int frame) {
		graphics.setColor(rgbLogo);
		if (frame > 100 - 8) {
			final int tmp = frame - (100 - 8) - 1;
			graphics.setColor(Graphics.getColorOfRGB(240 - tmp * 32, 240 - tmp * 24, 240 - tmp * 16));
		} else if (frame >= 28) {
			final int tmp = (frame % 28) % 24;
			if (0 <= tmp && tmp < 3) {
				graphics.setColor(Graphics.getColorOfRGB(240 - tmp * 64, 240 - tmp * 48, 240 - tmp * 32));
			}
		}

		for (int i = 0; i < pxLetters.length; i++) {
			graphics.setOrigin(pxLetterPosX[i], 48);
			for (int j = 0; j < pxLetters[i].length; j++) {
				graphics.drawPolyline(pxLetters[i][j][0], pxLetters[i][j][1], 0, pxLetters[i][j][0].length);
			}
		}
		graphics.setOrigin(0, 0);

		return (frame >= 100);
	}

	public static String getPictureName() {
		return "The Title of \"Dyna-Mo\"";
	}
}
